#
# spec file for package galene
#
# Copyright (c) 2023 SUSE LLC
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via https://bugs.opensuse.org/
#
# nodebuginfo


# Remove stripping of Go binaries.
%define __arch_install_post export NO_BRP_STRIP_DEBUG=true
# Project name when using go tooling.
%define project github.com/jech/galene
%bcond_without  apparmor
Name:           galene
Version:        0.7.1
Release:        bp156.1.14
Summary:        Galène videoconferencing server
License:        MIT
Group:          Development/Languages/Other
URL:            https://galene.org/
Source:         %{name}-%{version}.tar.gz
Source1:        vendor.tar.gz
Source2:        %{name}.service
Source3:        %{name}.sysconfig
Source4:        ice-servers.json
Source5:        apparmor-usr.sbin.galene
Patch1:         galene-html-sendselect-default.patch
BuildRequires:  fdupes
BuildRequires:  filesystem
BuildRequires:  go >= 1.16
BuildRequires:  systemd-rpm-macros
Requires:       fdupes
Requires:       filesystem
Requires(pre):  %fillup_prereq
%if %{with apparmor}
BuildRequires:  apparmor-abstractions
BuildRequires:  apparmor-rpm-macros
Recommends:     apparmor-abstractions
%endif

%description
Galène is a videoconferencing server implemented in Go which can be
deployed with moderate server resources.

%prep
%setup -qa1
%patch1 -p1

%build
%ifarch ppc64
go build -mod=vendor
%else
go build -mod=vendor -buildmode=pie
%endif

%install
%fdupes %{buildroot}/%{_prefix}
# Install the binary
mkdir -p %{buildroot}%{_sbindir}
install -D -m 0755 %{name} "%{buildroot}%{_sbindir}/%{name}"
# Install static web content
for file in $(find static -type f); do
  install -D -m 0644 $file %{buildroot}%{_datadir}/%{name}/$file
done
# Install systemd unit and symbolic link
mkdir -p %{buildroot}%{_unitdir}
install -p -m644 %{SOURCE2} %{buildroot}%{_unitdir}/
ln -s %{_sbindir}/service %{buildroot}%{_sbindir}/rc%{name}
# create config directory and create example config files
mkdir -p %{buildroot}%{_sysconfdir}/%{name}
install -m 0644 %{SOURCE4} %{buildroot}%{_sysconfdir}/%{name}/ice-servers.json.example
%if %{with apparmor}
# AppArmor profile
mkdir -p %{buildroot}%{_sysconfdir}/apparmor.d
install -m 0644 %{SOURCE5} %{buildroot}%{_sysconfdir}/apparmor.d/usr.sbin.galene
%endif
# create service directories
mkdir -p %{buildroot}%{_sharedstatedir}/%{name}/groups %{buildroot}%{_sharedstatedir}/%{name}/recordings
# create sysconfig file
mkdir -p %{buildroot}%{_fillupdir}
install -D -m 0644 %{SOURCE3} %{buildroot}%{_fillupdir}/sysconfig.%{name}

%pre
# creating group and user galene
%{_bindir}/getent group %{name} >/dev/null || %{_sbindir}/groupadd -r %{name}
%{_bindir}/getent passwd %{name} >/dev/null || \
  %{_sbindir}/useradd -r -g %{name} -d / -s /sbin/nologin \
  -c "%{name} service user" %{name}
%service_add_pre %{name}.service

%post
%service_add_post %{name}.service
%{fillup_only galene}
%if %{with apparmor}
%apparmor_reload %{_sysconfdir}/apparmor.d/usr.sbin.galene
%endif

%preun
%service_del_preun %{name}.service

%postun
%service_del_postun %{name}.service

%files
%doc CHANGES
%doc INSTALL
%doc README
%doc README.FRONTEND
%doc README.PROTOCOL
%license LICENCE
%{_sbindir}/%{name}
%dir %{_datadir}/%{name}
%dir %attr(0755, root, root) %{_datadir}/%{name}/static
%dir %attr(0755, root, root) %{_datadir}/%{name}/static/external/fontawesome
%dir %attr(0755, root, root) %{_datadir}/%{name}/static/external/fontawesome/css
%dir %attr(0755, root, root) %{_datadir}/%{name}/static/external/fontawesome/webfonts
%dir %attr(0755, root, root) %{_datadir}/%{name}/static/external/toastify
%dir %attr(0755, root, root) %{_datadir}/%{name}/static/external/contextual
%{_datadir}/%{name}/static/*
%{_datadir}/%{name}/static/external/fontawesome
%{_datadir}/%{name}/static/external/fontawesome/css/*.css
%{_datadir}/%{name}/static/external/fontawesome/webfonts/*
%{_datadir}/%{name}/static/external/toastify/*
%{_datadir}/%{name}/static/external/contextual/*
%{_sbindir}/rc%{name}
%{_unitdir}/%{name}.service
%dir %attr(0750, root, %{name}) %{_sysconfdir}/%{name}
%config %attr(0640, root, %{name}) %{_sysconfdir}/%{name}/ice-servers.json.example
%if %{with apparmor}
%dir %{_sysconfdir}/apparmor.d
%config %{_sysconfdir}/apparmor.d/usr.sbin.galene
%endif
%dir %attr(0750, root, %{name}) %{_sharedstatedir}/%{name}
%dir %attr(0570, %{name}, root) %{_sharedstatedir}/%{name}/groups
%dir %attr(0750, %{name}, %{name}) %{_sharedstatedir}/%{name}/recordings
%{_fillupdir}/sysconfig.%{name}

%changelog
* Thu Jun 15 2023 Andrea Manzini <andrea.manzini@suse.com>
- Update to version 0.7.1:
  * Fix incorrect error-handling in token parsing, which could cause
    Galene to crash after receiving a mis-formatted token.
  * Implement a dialog for creating tokens.
  * Tweak peer-to-peer file transfer to more reliably return an error
    message to the user.
  * Fix the workaround for non-standard simulcast in older Firefox
    versions.
- Update to version 0.7.0:
  * Updated the protocol version to 2.  Dates are now represented in
    ISO 8601 format, and there is a new "error" field in error messages
    that contains a machine-readable identifier.
  * Implemented stateful tokens ("invitations").
  * Made more errors non-fatal, which makes it possible for the client
    to recover in more situations.
  * Reworked the progress indicator used during file transfer.
  * Added Referrer-Policy and X-Content-Type-Options headers to HTTP
    responses (Galene was already generating HTML links with the
    "noreferrer" option).  Thanks to Alexandre Iooss.
  * Fixed a bug in the handling of leave messages.  (The bundled client
    never sends leave, it just disconnects, so the bug never triggerred.)
  * Changed the way simulcast tracks are selected to not depend on the rid.
- Update to version 0.6.2:
  * Disable simulcast for screensharing; this didn't work well with many
    browsers.
  * Fix parsing of VP8 packets with degenerate headers.
  * Fix computation of group URL when running in insecure mode.
  * Add configuration option "proxyURL", which makes it easier to run
    Galene behind a reverse proxy.
* Fri Oct  7 2022 michael@stroeder.com
- Update to version 0.6.1:
  * Add a camera/microphone indicator in the users list, and hide
    audio-only peers by default.  They can be reenabled in the side menu.
  * Ensure that autolocked groups are locked on creation.  Thanks to
    Michael Ströder.
  * Don't send chat history multiple times.  Thanks to Rémy Nollet.
  * Fix status display for remote (redirected) groups.
* Sat Sep 10 2022 michael@stroeder.com
- Update to version 0.6:
  * Version the protocol: the handshake message now contains a version
    number, and the server will warn if it is incorrect.  This will become
    a hard error in the future.
  * Rework the peer-to-peer file transfer protocol.  It now lives in
    protocol.js, which makes it easy to use by third-party clients.
  * Extend the authorization protocol to allow the authorization server to
    request that the client fall back to password authorization.
  * Make it possible to run the web server on a Unix domain socket.
    Thanks to Martin Pépin.
  * Check that a given group exists before redirecting from the landing
    page.  This avoids an unsightly 404 error when the user makes a typo.
  * Publish the group name end websocket endpoint in the group status.
    This avoids hardwirint the server structure in the client.
  * Fix a race condition that could cause chathistory messages to be sent
    before the joined status.
* Sat Jul 23 2022 michael@stroeder.com
- Update to version 0.5.5:
  * Fixed a longstanding bug with time conversion that would occasionally
    cause us to miscalculate rates and send incorrect sender reports.
    Thanks to lamhai1401.
  * Updated srtp to a version that includes Adrian Cable's optimisations.
    This should yield a 10%% to 20%% improvement for AES-CTR (but modern
    browsers use GCM).
  * Reworded the Safari warning to be more frightening, since
    screensharing on Safari tends to get stuck.
* Tue Jul 12 2022 Michael Ströder <michael@stroeder.com>
- explicitly require go >= 1.16
* Fri May  6 2022 michael@stroeder.com
- Update to version 0.5.4:
  * Fix username check when a token is present.
  * Improve the performance of AES in CTR mode.
  * Tweak detection of keyframes with H.264.
* Mon Apr 25 2022 michael@stroeder.com
- Update to version 0.5.3:
  * Remove a forgotten "replace" that prevented 0.5.2 from compiling
    unless "pion/ice" was downloaded manually.
  * Fix a race condition in the rate estimator.
  * Update pion/ice to a version that handles aggressive nomination; this
    avoids broken connection in some (fairly rare) cases.
* Wed Apr 20 2022 michael@stroeder.com
- Update to version 0.5.2:
  * Implement audio-video sync in the diskwriter.
  * Fix setting of the keyframe flag in audio samples in diskwriter.
  * Don't use high-quality audio for screenshare by default, obey the
    hqaudio preference in all cases.
  * Maintain user information for oneself, not just other users.
  * Call the onuser callback when a user deletes a stream.
* Fri Mar 25 2022 michael@stroeder.com
- Update to version 0.5.1:
  * Enable simulcast on Firefox.
  * Fix rate control on Firefox.
  * Fix corruption after packet reordering when saving to disk.
  * Fix the H.264 profile for compatibility with older iPhones.
  * Display a warning when joining a locked group.
  * Add a hook that allows the client to influence the ICE policy.
  * Fix error handling in peer-to-peer file transfer.
* Tue Mar  8 2022 michael@stroeder.com
- Update to version 0.5:
  * Implement third-party (token-based) authentication.
  * Implement peer-to-peer file transfer (/sendfile).
  * Add a contextual menu to the users list.
  * Removed the "Broadcast file" entry in the side menu, it is now
    available as a context menu entry and a chat command (/presentfile).
  * Add side-menu options for high-quality audio (Opus at 128kbps) and
    disabling audio preprocessing.
  * Send audio in addition to video when sharing browser tabs.
* Tue Feb  1 2022 michael@stroeder.com
- Update to version 0.4.4:
  * Update dependencies.  Pion WebRTC v3.1.19 fixes a bug that could cause
    audio tracks to be discarded under some circumstances.
  * Fix a bug that could cause user statuses (raisehand) to be discarded.
  * Fix incorrect maxRate value in statistics display.
* Thu Jan 27 2022 michael@stroeder.com
- Update to version 0.4.3
  * Fix a locking issue that could cause a deadlock when massive numbers
    of clients join at the same time.
  * Add support for displaying raised hands.
  * Update the MIME type for AV1 to track recent browsers.
  * Avoid overflow during rate estimation.
  * More fixes to the disk writer.
* Wed Dec  1 2021 michael@stroeder.com
- Update to version 0.4.2:
  * Update the samplebuilder to avoid a crash when a single frame is so
    large that it overflows the buffer.
  * Various other changes that improve the quality of disk recordings.
  * Update dependencies.
* Fri Nov  5 2021 Michael Ströder <michael@stroeder.com>
- adapted AppArmor profile to new config files in 0.4.1
* Fri Nov  5 2021 michael@stroeder.com
- Update to version 0.4.1:
  * Create a new file data/config.json with global configuration.
  * Remove data/passwd and the -redirect option with entries in config.json;
    these are incompatible changes.
  * Change the group URL from /group/name to /group/name/, which allows
    using relative links.  The old URL is redirected to the new one.
  * Extend the protocol with the ability to consult the group status before
    joining; this allows using the group's displayName before the user has
    logged in, and will be required for new authorisation schemes.
  * Allow scrolling of the login dialog, useful on small devices.
  * Fixed a typo that prevented the group name from being displayed.
  * Made failed videos more visible.
  * No longer attempt to save passwords in browser JavaScript, it's
    insecure and not very useful.
* Mon Sep  6 2021 Michael Ströder <michael@stroeder.com>
- avoid switching off IPv6
* Sun Sep  5 2021 michael@stroeder.com
- Update to version 0.4:
  * Implemented simulcast.
  * Implemented scalable video coding (SVC) for VP8 and VP9.
  * Preliminary support for the AV1 codec.
  * Fixed the profile announced for VP9.
  * Reworked the disk writer, which should no longer cause corruption
    after packet loss.
  * Added support for recording H.264 tracks.
  * Reworked the signalling protocol, it is now possible to request
    different tracks for distinct streams, and there are provisions to
    carry user permissions and statuses; this is not yet used by the user
    interface.
  * Reworked internal signalling; this will hopefully fix issues with
    losing tracks under load.
  * Added an option to limit the range of UDP ports used by the server.
  * Added a close button in every locally-generated video; this allows
    closing individual screen shares and simplifies the UI by not creating
    close buttons in the top bar.
  * The server will now attempt to increase the file descriptor limit at
    startup.
  * Stats are now exported as JSON and formatted on the client side; the
    stats URL is now /stats.html rather than just /stats.
* Mon Jun  7 2021 Michael Ströder <michael@stroeder.com>
- removed madvise from SystemCallFilter=
* Wed May 12 2021 Michael Ströder <michael@stroeder.com>
- Update to version 0.3.5:
  * Fixed another bug in the loss estimator.
  * Fixed the list of SSRCs sent in REMB packets.  This doesn't seem to
    have any measurable effect.
  * Removed some obsolete code that caused clients to be kicked out when
    they lost the Present permission.
* Tue Apr 27 2021 michael@stroeder.com
- Update to version 0.3.4:
  * Fixed a bug in the loss estimator.  We will now reduce the rate more
    aggressively when there is loss in the client->serer direction.
  * Fixed display of the chat on some devices.
  * Exported the group.Description data structure for the sake of external
    configuration tools.
* Fri Apr 16 2021 michael@stroeder.com
- Update to version 0.3.3:
  * Fixed an off-by-one error that could occasionally lead to a server
    crash.
  * Attempt to negotiate stereo Opus audio.
* Wed Mar  3 2021 Michael Ströder <michael@stroeder.com>
- build without PIE on PPC64 platform
* Tue Mar  2 2021 michael@stroeder.com
- Update to version 0.3.2:
  * Fixed compatibility with versions of Go earlier than 1.15.
  * Fixed a race condition in the disk writer.
* Fri Feb 26 2021 michael@stroeder.com
- Update to version 0.3.1:
  * We now notice when the TLS certificate has changed on disk; which can
    therefore be rotated without a restart.
  * We now generate a self-signed certificate if none is found on disk.
* Tue Feb 23 2021 Michael Ströder <michael@stroeder.com>
- installation of AppArmor profile truly optional
- dropped version check for AppArmor dependencies
* Thu Feb 18 2021 Michael Ströder <michael@stroeder.com>
- added CHANGES
- added galene-html-sendselect-default.patch
- Update to version 0.3:
  * Implemented the command /muteall.  This allows muting all users except
    yourself.
  * Implemented the autolock and autokick user group options, that cause,
    a group to be locked or all users to be kicked out when the last
    administrator leaves.
  * Implemented client-side filters that are run over the video before it
    is sent to the server.  A pair of mirroring filters are provided,
    which are useful when using a mirror in front of a webcam.
  * Included a built-in TURN server, which drastically simplifies
    deployment in common cases.
  * Implemented testing of TURN servers, on both the client and the server
    side.  Connectivity will be checked by the server every 12 hours, and
    by the client whenever a user uses the /relay-test command.
  * Group configurations are now checked at startup, and any failures are
    logged.  This implies that it is no longer possible to use undefined
    keys in group definition files (this is a breaking change).
  * Reworked the client API, which is now simpler and more uniform.
  * Subgroups can now be public.
  * Added an option to disable the mirror view, which is useful when writing.
  * Ported file playback to Firefox.
  * We now display a friendly error message when the user has disabled
    WebRTC under Firefox.
  * Implemented renegotiation of down streams, which reduces the amount of
    flicker in the user interface.
  * Fixed a number of race conditions that could cause audio tracks to
    become muted or video tracks to freeze.
* Mon Jan 25 2021 Michael Ströder <michael@stroeder.com>
- added galene-html-sendselect-low.patch
* Mon Jan 18 2021 Michael Ströder <michael@stroeder.com>
- do not install non-functional /etc/galene/ice-servers.json
* Mon Jan 18 2021 Michael Ströder <michael@stroeder.com>
- added AppArmor profile
* Mon Jan 18 2021 Michael Ströder <michael@stroeder.com>
- added ice-servers.json example
- fixed static/* files
- added galene.service with hardening options
- added /etc/sysconfig/galene
- create directories for config, groups and recordings
* Mon Jan 18 2021 Jeff Kowalczyk <jkowalczyk@suse.com>
- Fix usage of obs-service-go_modules with upstream version spelling
  * Remove unused dependency on golang-packaging
  * Fixes %%install usage with static content directory
  * Remove duplicated generated changelog entries including manual
    removal of upstream housekeeping commits e.g. update README, etc.
  * Use %%license macro with upstream file spelling LICENCE
* Sat Jan 16 2021 michael@stroeder.com
- Update to version 0.2:
  * Reduce ping timetout to 45 to 55s.
  * Fix typos.
  * Fix removal of stream.onremovetrack callback.
  * Add missing semicolons.
  * Call ondowntrack on each track.
  * Add websocket handshake timeout.
  * Optimise broadcast messages.
* Fri Jan 15 2021 Michael Ströder <michael@stroeder.com>
- initial package of release 0.2
